@@ -1654,11 +1654,11 @@ RescanMenuItem.prototype = {
 };
 
 
-function MyApplet(metadata, orientation, panel_height, instance_id) {
+function CinnamonNetworkApplet(metadata, orientation, panel_height, instance_id) {
     this._init(metadata, orientation, panel_height, instance_id);
 }
 
-MyApplet.prototype = {
+CinnamonNetworkApplet.prototype = {
     __proto__: Applet.IconApplet.prototype,
 
     _init: function(metadata, orientation, panel_height, instance_id) {
@@ -2322,6 +2322,5 @@ MyApplet.prototype = {
 };
 
 function main(metadata, orientation, panel_height, instance_id) {
-    let myApplet = new MyApplet(metadata, orientation, panel_height, instance_id);
-    return myApplet;
+    return new CinnamonNetworkApplet(metadata, orientation, panel_height, instance_id);
 }
