@@ -1,8 +1,6 @@
-#!/usr/bin/python2
+#!/usr/bin/python3
 
 import sys
-reload(sys)
-sys.setdefaultencoding('utf-8')
 
 import os
 import glob
@@ -10,31 +8,33 @@ import gettext
 import time
 import traceback
 import locale
-import urllib2
+import urllib.request as urllib
 from functools import cmp_to_key
 import unicodedata
+import config
 
 import gi
 gi.require_version('Gtk', '3.0')
-from gi.repository import Gio, Gtk, Pango, Gdk
+gi.require_version('XApp', '1.0')
+from gi.repository import Gio, Gtk, Pango, Gdk, XApp
 
-sys.path.append('/usr/share/cinnamon/cinnamon-settings/modules')
-sys.path.append('/usr/share/cinnamon/cinnamon-settings/bin')
+sys.path.append(config.currentPath + "/modules")
+sys.path.append(config.currentPath + "/bin")
 import capi
 import proxygsettings
 import SettingsWidgets
 
 # i18n
-gettext.install("cinnamon", "/usr/share/locale")
+gettext.install("cinnamon", "/usr/share/locale", names="ngettext")
 
 # Standard setting pages... this can be expanded to include applet dirs maybe?
-mod_files = glob.glob('/usr/share/cinnamon/cinnamon-settings/modules/*.py')
+mod_files = glob.glob(config.currentPath + "/modules/*.py")
 mod_files.sort()
 if len(mod_files) is 0:
-    print "No settings modules found!!"
+    print("No settings modules found!!")
     sys.exit(1)
 
-mod_files = [x.split('/')[6].split('.')[0] for x in mod_files]
+mod_files = [x.split('/')[-1].split('.')[0] for x in mod_files]
 
 for mod_file in mod_files:
     if mod_file[0:3] != "cs_":
@@ -58,7 +58,7 @@ MAX_PIX_WIDTH = 160
 MOUSE_BACK_BUTTON = 8
 
 CATEGORIES = [
-#        Display name                         ID              Show it? Always False to start              Icon
+    #        Display name                         ID              Show it? Always False to start              Icon
     {"label": _("Appearance"),            "id": "appear",      "show": False,                       "icon": "cs-cat-appearance"},
     {"label": _("Preferences"),           "id": "prefs",       "show": False,                       "icon": "cs-cat-prefs"},
     {"label": _("Hardware"),              "id": "hardware",    "show": False,                       "icon": "cs-cat-hardware"},
@@ -66,7 +66,7 @@ CATEGORIES = [
 ]
 
 CONTROL_CENTER_MODULES = [
-#         Label                              Module ID                Icon                         Category      Keywords for filter
+    #         Label                              Module ID                Icon                         Category      Keywords for filter
     [_("Network"),                          "network",            "cs-network",                 "hardware",      _("network, wireless, wifi, ethernet, broadband, internet")],
     [_("Display"),                          "display",            "cs-display",                 "hardware",      _("display, screen, monitor, layout, resolution, dual, lcd")],
     [_("Color"),                            "color",              "cs-color",                   "hardware",      _("color, profile, display, printer, output")],
@@ -74,16 +74,23 @@ CONTROL_CENTER_MODULES = [
 ]
 
 STANDALONE_MODULES = [
-#         Label                          Executable                          Icon                Category        Keywords for filter
-    [_("Printers"),                      "system-config-printer",        "cs-printer",         "hardware",       _("printers, laser, inkjet")],
-    [_("Firewall"),                      "gufw",                         "cs-firewall",        "admin",          _("firewall, block, filter, programs")],
-    [_("Languages"),                     "mintlocale",                   "cs-language",        "prefs",          _("language, install, foreign")],
-    [_("Input Method"),                  "mintlocale im",                "cs-input-method",    "prefs",          _("language, install, foreign, input, method, chinese, korean, japanese, typing")],
-    [_("Login Window"),                  "gksu /usr/sbin/mdmsetup",      "cs-login",           "admin",          _("login, mdm, gdm, manager, user, password, startup, switch")],
-    [_("Driver Manager"),                "mintdrivers",                  "cs-drivers",         "admin",          _("video, driver, wifi, card, hardware, proprietary, nvidia, radeon, nouveau, fglrx")],
-    [_("Software Sources"),              "mintsources",                  "cs-sources",         "admin",          _("ppa, repository, package, source, download")],
-    [_("Users and Groups"),              "cinnamon-settings-users",      "cs-user-accounts",   "admin",          _("user, users, account, accounts, group, groups, password")],
-    [_("Bluetooth"),                     "blueberry",                    "cs-bluetooth",       "hardware",       _("bluetooth, dongle, transfer, mobile")]
+    #         Label                          Executable                          Icon                Category        Keywords for filter
+    [_("Printers"),                      "system-config-printer",               "cs-printer",         "hardware",       _("printers, laser, inkjet")],
+    [_("Firewall"),                      "gufw",                                "cs-firewall",        "admin",          _("firewall, block, filter, programs")],
+    [_("Firewall"),                      "firewall-config",                     "cs-firewall",        "admin",          _("firewall, block, filter, programs")],
+    [_("Languages"),                     "mintlocale",                          "cs-language",        "prefs",          _("language, install, foreign")],
+    [_("Input Method"),                  "mintlocale im",                       "cs-input-method",    "prefs",          _("language, install, foreign, input, method, chinese, korean, japanese, typing")],
+    [_("Login Window"),                  "gksu /usr/sbin/mdmsetup",             "cs-login",           "admin",          _("login, mdm, gdm, manager, user, password, startup, switch")],
+    [_("Login Window"),                  "lightdm-settings",                    "cs-login",           "admin",          _("login, lightdm, mdm, gdm, manager, user, password, startup, switch")],
+    [_("Login Window"),                  "lightdm-gtk-greeter-settings-pkexec", "cs-login",           "admin",          _("login, lightdm, manager, settings, editor")],
+    [_("Driver Manager"),                "mintdrivers",                         "cs-drivers",         "admin",          _("video, driver, wifi, card, hardware, proprietary, nvidia, radeon, nouveau, fglrx")],
+    [_("Nvidia Settings"),               "nvidia-settings",                     "cs-drivers",         "admin",          _("video, driver, proprietary, nvidia, settings")],
+    [_("Software Sources"),              "mintsources",                         "cs-sources",         "admin",          _("ppa, repository, package, source, download")],
+    [_("Package Management"),            "dnfdragora",                          "cs-sources",         "admin",          _("update, install, repository, package, source, download")],
+    [_("Package Management"),            "yumex-dnf",                           "cs-sources",         "admin",          _("update, install, repository, package, source, download")],
+    [_("Users and Groups"),              "cinnamon-settings-users",             "cs-user-accounts",   "admin",          _("user, users, account, accounts, group, groups, password")],
+    [_("Bluetooth"),                     "blueberry",                           "cs-bluetooth",       "hardware",       _("bluetooth, dongle, transfer, mobile")],
+    [_("Manage Services and Units"),     "systemd-manager-pkexec",              "cs-sources",         "admin",          _("systemd, units, services, systemctl, init")]
 ]
 
 def print_timing(func):
@@ -92,7 +99,7 @@ def print_timing(func):
         t1 = time.time()
         res = func(*arg)
         t2 = time.time()
-        print '%s took %0.3f ms' % (func.func_name, (t2-t1)*1000.0)
+        print('%s took %0.3f ms' % (func.func_name, (t2-t1)*1000.0))
         return res
     return wrapper
 
@@ -122,6 +129,7 @@ class MainWindow:
         sidePage = self.store[cat].get_value(iterator,2)
         if not sidePage.is_standalone:
             self.window.set_title(sidePage.name)
+            self.window.set_icon_name(sidePage.icon)
             sidePage.build()
             if sidePage.stack:
                 current_page = sidePage.stack.get_visible_child_name()
@@ -155,11 +163,6 @@ class MainWindow:
     def maybe_resize(self, sidePage):
         m, n = self.content_box.get_preferred_size()
 
-        # Resize horizontally if the module is wider than the window
-        use_width = WIN_WIDTH
-        if n.width > WIN_WIDTH:
-            use_width = n.width
-
         # Resize vertically depending on the height requested by the module
         use_height = WIN_HEIGHT
         total_height = n.height + self.bar_heights + WIN_H_PADDING
@@ -167,7 +170,6 @@ class MainWindow:
             # No height requested, resize vertically if the module is taller than the window
             if total_height > WIN_HEIGHT:
                 use_height = total_height
-            #self.window.resize(use_width, n.height + self.bar_heights + WIN_H_PADDING)
         elif sidePage.size > 0:
             # Height hardcoded by the module
             use_height = sidePage.size + self.bar_heights + WIN_H_PADDING
@@ -175,18 +177,21 @@ class MainWindow:
             # Module requested the window to fit it (i.e. shrink the window if necessary)
             use_height = total_height
 
-        self.window.resize(use_width, use_height)
+        self.window.resize(WIN_WIDTH, use_height)
 
     def deselect(self, cat):
-        for key in self.side_view.keys():
+        for key in self.side_view:
             if key is not cat:
                 self.side_view[key].unselect_all()
 
     ''' Create the UI '''
     def __init__(self):
         self.builder = Gtk.Builder()
-        self.builder.add_from_file("/usr/share/cinnamon/cinnamon-settings/cinnamon-settings.ui")
-        self.window = self.builder.get_object("main_window")
+        self.builder.add_from_file(config.currentPath + "/cinnamon-settings.ui")
+        self.window = XApp.GtkWindow(window_position=Gtk.WindowPosition.CENTER,
+                                     default_width=800, default_height=600)
+        main_box = self.builder.get_object("main_box")
+        self.window.add(main_box)
         self.top_bar = self.builder.get_object("top_bar")
         self.side_view = {}
         self.main_stack = self.builder.get_object("main_stack")
@@ -215,6 +220,7 @@ class MainWindow:
         self.stack_switcher.set_margin_right(n)
 
         self.search_entry = self.builder.get_object("search_box")
+        self.search_entry.set_placeholder_text(_("Search"))
         self.search_entry.connect("changed", self.onSearchTextChanged)
         self.search_entry.connect("icon-press", self.onClearSearchBox)
 
@@ -241,7 +247,7 @@ class MainWindow:
                 if self.loadCheck(mod) and self.setParentRefs(mod):
                     self.unsortedSidePages.append((mod.sidePage, mod.name, mod.category))
             except:
-                print "Failed to load module %s" % module
+                print("Failed to load module %s" % module)
                 traceback.print_exc()
 
         for item in CONTROL_CENTER_MODULES:
@@ -255,15 +261,10 @@ class MainWindow:
                 self.unsortedSidePages.append((samodule.sidePage, samodule.name, samodule.category))
 
         # sort the modules alphabetically according to the current locale
-        sidePageNamesToSort = map(lambda m: m[0].name, self.unsortedSidePages)
-        sortedSidePageNames = sorted(sidePageNamesToSort, key=cmp_to_key(locale.strcoll))
-        for sidePageName in sortedSidePageNames:
-            nextSidePage = None
-            for trySidePage in self.unsortedSidePages:
-                if(trySidePage[0].name == sidePageName):
-                    nextSidePage = trySidePage
-
-            self.sidePages.append(nextSidePage);
+        localeStrKey = cmp_to_key(locale.strcoll)
+        # Apply locale key to the field name of each side page.
+        sidePagesKey = lambda m : localeStrKey(m[0].name)
+        self.sidePages = sorted(self.unsortedSidePages, key=sidePagesKey)
 
 
         # create the backing stores for the side nav-view.
@@ -272,14 +273,14 @@ class MainWindow:
         self.storeFilter = {}
         for sidepage in self.sidePages:
             sp, sp_id, sp_cat = sidepage
-            if not self.store.has_key(sp_cat):  #       Label         Icon          sidePage     Category
+            if sp_cat not in self.store:        #       Label         Icon    sidePage    Category
                 self.store[sidepage[2]] = Gtk.ListStore(str,          str,    object,     str)
                 for category in CATEGORIES:
                     if category["id"] == sp_cat:
                         category["show"] = True
 
             # Don't allow item names (and their translations) to be more than 30 chars long. It looks ugly and it creates huge gaps in the icon views
-            name = unicode(sp.name,'utf-8')
+            name = sp.name
             if len(name) > 30:
                 name = "%s..." % name[:30]
             sidePagesIters[sp_id] = (self.store[sp_cat].append([name, sp.icon, sp, sp_cat]), sp_cat)
@@ -287,7 +288,7 @@ class MainWindow:
         self.min_label_length = 0
         self.min_pix_length = 0
 
-        for key in self.store.keys():
+        for key in self.store:
             char, pix = self.get_label_min_width(self.store[key])
             self.min_label_length = max(char, self.min_label_length)
             self.min_pix_length = max(pix, self.min_pix_length)
@@ -312,7 +313,7 @@ class MainWindow:
         self.calculate_bar_heights()
 
         # Select the first sidePage
-        if len(sys.argv) > 1 and sys.argv[1] in sidePagesIters.keys():
+        if len(sys.argv) > 1 and sys.argv[1] in sidePagesIters:
             (iter, cat) = sidePagesIters[sys.argv[1]]
             path = self.store[cat].get_path(iter)
             if path:
@@ -328,7 +329,7 @@ class MainWindow:
         if device.get_source() == Gdk.InputSource.KEYBOARD:
             grab = Gdk.Display.get_default().device_is_grabbed(device)
         if not grab and event.keyval == Gdk.KEY_BackSpace and (type(self.window.get_focus()) not in
-                    (Gtk.TreeView, Gtk.Entry, Gtk.SpinButton, Gtk.TextView)):
+                                                               (Gtk.TreeView, Gtk.Entry, Gtk.SpinButton, Gtk.TextView)):
             self.back_to_icon_view(None)
             return True
         return False
@@ -576,6 +577,7 @@ class MainWindow:
 
     def back_to_icon_view(self, widget):
         self.window.set_title(_("System Settings"))
+        self.window.set_icon_name("preferences-system")
         self.window.resize(WIN_WIDTH, WIN_HEIGHT)
         children = self.content_box.get_children()
         for child in children:
@@ -598,10 +600,10 @@ if __name__ == "__main__":
 
     ps = proxygsettings.get_proxy_settings()
     if ps:
-        proxy = urllib2.ProxyHandler(ps)
+        proxy = urllib.ProxyHandler(ps)
     else:
-        proxy = urllib2.ProxyHandler()
-    urllib2.install_opener(urllib2.build_opener(proxy))
+        proxy = urllib.ProxyHandler()
+    urllib.install_opener(urllib.build_opener(proxy))
 
     window = MainWindow()
     signal.signal(signal.SIGINT, window.quit)
