@@ -1,4 +1,4 @@
-#!/usr/bin/python2
+#!/usr/bin/python3
 
 from GSettingsWidgets import *
 from gi.repository import *
@@ -39,78 +39,78 @@ preferred_app_defs = [
 
 mimetypes = {}
 mimetypes["audio"]=[
-  "audio/3gpp",
-  "audio/aac",
-  "audio/ac3",
-  "audio/flac",
-  "audio/m4a",
-  "audio/midi",
-  "audio/mp3",
-  "audio/mp4",
-  "audio/mp4a-latm",
-  "audio/mpeg",
-  "audio/mpeg3",
-  "audio/mpg",
-  "audio/ogg",
-  "audio/vorbis",
-  "audio/wav",
-  "audio/wave",
-  "audio/webm",
-  "audio/x-aac",
-  "audio/x-aiff",
-  "audio/x-flac",
-  "audio/x-mp3",
-  "audio/x-mpeg",
-  "audio/x-mpeg-3",
-  "audio/x-mpg",
-  "audio/x-ms-asf",
-  "audio/x-ms-wma",
-  "audio/x-ogg",
-  "audio/x-oggflac",
-  "audio/x-vorbis",
-  "audio/x-vorbis+ogg",
-  "audio/x-wav",
-  "audio/x-wavpack"
+    "audio/3gpp",
+    "audio/aac",
+    "audio/ac3",
+    "audio/flac",
+    "audio/m4a",
+    "audio/midi",
+    "audio/mp3",
+    "audio/mp4",
+    "audio/mp4a-latm",
+    "audio/mpeg",
+    "audio/mpeg3",
+    "audio/mpg",
+    "audio/ogg",
+    "audio/vorbis",
+    "audio/wav",
+    "audio/wave",
+    "audio/webm",
+    "audio/x-aac",
+    "audio/x-aiff",
+    "audio/x-flac",
+    "audio/x-mp3",
+    "audio/x-mpeg",
+    "audio/x-mpeg-3",
+    "audio/x-mpg",
+    "audio/x-ms-asf",
+    "audio/x-ms-wma",
+    "audio/x-ogg",
+    "audio/x-oggflac",
+    "audio/x-vorbis",
+    "audio/x-vorbis+ogg",
+    "audio/x-wav",
+    "audio/x-wavpack"
 ]
 
 mimetypes["video"]=[
-  "video/3gp",
-  "video/3gpp",
-  "video/divx",
-  "video/flv",
-  "video/mp4",
-  "video/mp4v-es",
-  "video/mpeg",
-  "video/msvideo",
-  "video/ogg",
-  "video/quicktime",
-  "video/vivo",
-  "video/vnd.divx",
-  "video/vnd.rn-realvideo",
-  "video/webm",
-  "video/x-anim",
-  "video/x-avi",
-  "video/x-flc",
-  "video/x-fli",
-  "video/x-flic",
-  "video/x-flv",
-  "video/x-m4v",
-  "video/x-matroska",
-  "video/x-mng",
-  "video/x-mpeg",
-  "video/x-mpeg2",
-  "video/x-ms-afs",
-  "video/x-ms-asf",
-  "video/x-ms-asx",
-  "video/x-ms-wm",
-  "video/x-ms-wmv",
-  "video/x-ms-wvx",
-  "video/x-ms-wvxvideo",
-  "video/x-msvideo",
-  "video/x-nsv",
-  "video/x-ogm+ogg",
-  "video/x-theora",
-  "video/x-theora+ogg"
+    "video/3gp",
+    "video/3gpp",
+    "video/divx",
+    "video/flv",
+    "video/mp4",
+    "video/mp4v-es",
+    "video/mpeg",
+    "video/msvideo",
+    "video/ogg",
+    "video/quicktime",
+    "video/vivo",
+    "video/vnd.divx",
+    "video/vnd.rn-realvideo",
+    "video/webm",
+    "video/x-anim",
+    "video/x-avi",
+    "video/x-flc",
+    "video/x-fli",
+    "video/x-flic",
+    "video/x-flv",
+    "video/x-m4v",
+    "video/x-matroska",
+    "video/x-mng",
+    "video/x-mpeg",
+    "video/x-mpeg2",
+    "video/x-ms-afs",
+    "video/x-ms-asf",
+    "video/x-ms-asx",
+    "video/x-ms-wm",
+    "video/x-ms-wmv",
+    "video/x-ms-wvx",
+    "video/x-ms-wvxvideo",
+    "video/x-msvideo",
+    "video/x-nsv",
+    "video/x-ogm+ogg",
+    "video/x-theora",
+    "video/x-theora+ogg"
 ]
 
 mimetypes["text/x-python"] = [
@@ -207,11 +207,11 @@ class DefaultAppChooserButton(Gtk.AppChooserButton):
 
     def onChanged(self, button):
         info = button.get_app_info()
-        print "%s: " % info.get_name()
+        print("%s: " % info.get_name())
         if info:
             supported_mimetypes = info.get_supported_types()
             hardcoded_mimetypes = None
-            if self.generic_content_type in mimetypes.keys():
+            if self.generic_content_type in mimetypes:
                 hardcoded_mimetypes = mimetypes[self.generic_content_type]
 
             set_mimes = []
@@ -221,24 +221,24 @@ class DefaultAppChooserButton(Gtk.AppChooserButton):
                 for t in sorted(supported_mimetypes):
                     if t.startswith(self.generic_content_type):
                         if info.set_as_default_for_type (t):
-                            print "  Set as default for supported %s" % t
+                            print("  Set as default for supported %s" % t)
                             set_mimes.append(t)
                         else:
-                            print "  Failed to set as default application for '%s'" % t
+                            print("  Failed to set as default application for '%s'" % t)
 
             # Also assign mimes hardcoded in the mimetypes hashtable
             if hardcoded_mimetypes is not None:
                 for t in sorted(hardcoded_mimetypes):
                     if t not in set_mimes:
                         if info.set_as_default_for_type (t):
-                            print "  Set as default for hardcoded %s" % t
+                            print("  Set as default for hardcoded %s" % t)
                         else:
-                            print "  Failed to set as default application for '%s'" % t
+                            print("  Failed to set as default application for '%s'" % t)
 
             #Web
             if self.content_type == "x-scheme-handler/http":
                 if info.set_as_default_for_type ("x-scheme-handler/https") == False:
-                    print "  Failed to set '%s' as the default application for '%s'" % (info.get_name(), "x-scheme-handler/https")
+                    print("  Failed to set '%s' as the default application for '%s'" % (info.get_name(), "x-scheme-handler/https"))
 
 class DefaultTerminalButton(Gtk.AppChooserButton): #TODO: See if we can get this to change the x-terminal-emulator default to allow it to be a more global change rather then just cinnamon/nemo
     def __init__(self):
@@ -353,9 +353,10 @@ class CustomAppChooserButton(Gtk.AppChooserButton):
         self.setPreference(pref_open_folder, PREF_MEDIA_AUTORUN_X_CONTENT_OPEN_FOLDER)
 
 class OtherTypeDialog(Gtk.Dialog):
-    def __init__(self, media_settings):
+    def __init__(self, media_settings, transient_parent):
         super(OtherTypeDialog, self).__init__(title = _("Other Media"),
-                                              transient_for = None,
+                                              transient_for = transient_parent,
+                                              border_width = 6,
                                               flags = 0)
         self.add_button(_("Close"), Gtk.ResponseType.OK)
 
@@ -408,7 +409,7 @@ class OtherTypeDialog(Gtk.Dialog):
                 break
 
         if description == None:
-            print "Content type '%s' is missing from the info panel" % content_type
+            print("Content type '%s' is missing from the info panel" % content_type)
             return Gio.content_type_get_description(content_type)
 
         return description
@@ -469,10 +470,10 @@ class Module:
 
     def on_module_selected(self):
         if not self.loaded:
-            print "Loading Default module"
+            print("Loading Default module")
 
             self.media_settings = Gio.Settings.new(MEDIA_HANDLING_SCHEMA)
-            self.other_type_dialog = OtherTypeDialog(self.media_settings)
+            self.other_type_dialog = OtherTypeDialog(self.media_settings, self.sidePage.window)
 
             self.sidePage.stack = SettingsStack()
             self.sidePage.add_widget(self.sidePage.stack)
@@ -531,12 +532,16 @@ class Module:
                 widget.pack_end(button, False, False, 0)
                 settings.add_row(widget)
 
-            button = Button(_("_Other Media..."), self.onMoreClicked)
+            # FIXMEEEEEEEE??
+            button = Button(_("_Other Media...").strip("_"), self.onMoreClicked)
             settings.add_row(button)
 
     def onMoreClicked(self, widget):
         self.other_type_dialog.doShow(widget.get_toplevel())
 
+    def _setParentRef(self, window):
+        self.sidePage.window = window
+
 class InvertedSwitch(SettingsWidget):
     def __init__(self, label, schema, key):
         self.key = key
