@@ -1,11 +1,16 @@
-#!/usr/bin/env python2
+#!/usr/bin/python3
 
-from SettingsWidgets import *
 from gi.repository import Gio, Gtk, GObject, Gdk
-from KeybindingWidgets import CellRendererKeybinding
 import cgi
 import gettext
 
+import gi
+gi.require_version("Gtk", "3.0")
+from gi.repository import Gio, Gtk, GObject, Gdk
+
+from KeybindingWidgets import CellRendererKeybinding
+from GSettingsWidgets import *
+
 gettext.install("cinnamon", "/usr/share/locale")
 
 # Keybindings page - check if we need to store custom
@@ -20,26 +25,26 @@ CINNAMON_SCHEMA = "org.cinnamon.desktop.keybindings"
 
 CATEGORIES = [
 
-#   Label                   id                  parent
-#(child)Label                       id                  parent
+    #   Label                   id                  parent
+    #(child)Label                       id                  parent
 
     [_("General"),          "general",          None,       "preferences-desktop-keyboard-shortcuts"],
-        [_("Troubleshooting"),      "trouble",          "general",      None],
+    [_("Troubleshooting"),      "trouble",          "general",      None],
     [_("Windows"),          "windows",          None,       "preferences-system-windows"],
-        [_("Positioning"),          "win-position",     "windows",      None],
-        [_("Tiling and Snapping"),  "win-tiling",       "windows",      None],
-        [_("Inter-workspace"),      "win-workspaces",   "windows",      None],
-        [_("Inter-monitor"),        "win-monitors",     "windows",      None],
+    [_("Positioning"),          "win-position",     "windows",      None],
+    [_("Tiling and Snapping"),  "win-tiling",       "windows",      None],
+    [_("Inter-workspace"),      "win-workspaces",   "windows",      None],
+    [_("Inter-monitor"),        "win-monitors",     "windows",      None],
     [_("Workspaces"),       "workspaces",       None,       "video-display"],
-        [_("Direct Navigation"),    "ws-navi",          "workspaces",   None],
+    [_("Direct Navigation"),    "ws-navi",          "workspaces",   None],
     [_("System"),           "system",           None,       "preferences-system"],
-        [_("Hardware"),             "sys-hw",           "system",       None],
-        [_("Screenshots and Recording"),"sys-screen",   "system",       None],
+    [_("Hardware"),             "sys-hw",           "system",       None],
+    [_("Screenshots and Recording"),"sys-screen",   "system",       None],
     [_("Launchers"),        "launchers",        None,       "applications-utilities"],
     [_("Sound and Media"),  "media",            None,       "applications-multimedia"],
-        [_("Quiet Keys"),           "media-quiet",      "media",        None],
+    [_("Quiet Keys"),           "media-quiet",      "media",        None],
     [_("Universal Access"), "accessibility",    None,       "preferences-desktop-accessibility"],
-    [_("Custom Shortcuts"), "custom",           None,       "system-run"]
+    [_("Custom Shortcuts"), "custom",           None,       "cinnamon-panel-launcher"]
 ]
 
 KEYBINDINGS = [
@@ -148,6 +153,7 @@ KEYBINDINGS = [
     # System - Hardware
     [_("Re-detect display devices"), MEDIA_KEYS_SCHEMA, "video-outputs", "sys-hw"],
     [_("Rotate display"), MEDIA_KEYS_SCHEMA, "video-rotation", "sys-hw"],
+    [_("Orientation Lock"), MEDIA_KEYS_SCHEMA, "video-rotation-lock", "sys-hw"],
     [_("Increase screen brightness"), MEDIA_KEYS_SCHEMA, "screen-brightness-up", "sys-hw"],
     [_("Decrease screen brightness"), MEDIA_KEYS_SCHEMA, "screen-brightness-down", "sys-hw"],
     [_("Toggle keyboard backlight"), MEDIA_KEYS_SCHEMA, "kbd-brightness-toggle", "sys-hw"],
@@ -182,8 +188,8 @@ KEYBINDINGS = [
     [_("Shuffle"), MEDIA_KEYS_SCHEMA, "audio-random", "media"],
     # Sound and Media Quiet
     [_("Volume mute (Quiet)"), MEDIA_KEYS_SCHEMA, "mute-quiet", "media-quiet"],    # Not sure this is even necessary
-    [_("Volume down (Quiet)"), MEDIA_KEYS_SCHEMA, "volume-down", "media-quiet"],
-    [_("Volume up (Quiet)"), MEDIA_KEYS_SCHEMA, "volume-up", "media-quiet"],
+    [_("Volume down (Quiet)"), MEDIA_KEYS_SCHEMA, "volume-down-quiet", "media-quiet"],
+    [_("Volume up (Quiet)"), MEDIA_KEYS_SCHEMA, "volume-up-quiet", "media-quiet"],
     # Universal Access
     [_("Zoom in"), CINNAMON_SCHEMA, "magnifier-zoom-in", "accessibility"],
     [_("Zoom out"), CINNAMON_SCHEMA, "magnifier-zoom-out", "accessibility"],
@@ -206,7 +212,7 @@ class Module:
 
     def on_module_selected(self):
         if not self.loaded:
-            print "Loading Keyboard module"
+            print("Loading Keyboard module")
 
             self.sidePage.stack = SettingsStack()
             self.sidePage.add_widget(self.sidePage.stack)
@@ -222,10 +228,10 @@ class Module:
             switch = GSettingsSwitch(_("Enable key repeat"), "org.cinnamon.settings-daemon.peripherals.keyboard", "repeat")
             settings.add_row(switch)
 
-            slider = GSettingsRange(_("Repeat delay:"), "org.cinnamon.settings-daemon.peripherals.keyboard", "delay", _("Short"), _("Long"), 100, 2000)
+            slider = GSettingsRange(_("Repeat delay:"), "org.cinnamon.settings-daemon.peripherals.keyboard", "delay", _("Short"), _("Long"), 100, 2000, show_value=False)
             settings.add_reveal_row(slider, "org.cinnamon.settings-daemon.peripherals.keyboard", "repeat")
 
-            slider = GSettingsRange(_("Repeat speed:"), "org.cinnamon.settings-daemon.peripherals.keyboard", "repeat-interval", _("Slow"), _("Fast"), 20, 2000, invert=True, log=True)
+            slider = GSettingsRange(_("Repeat speed:"), "org.cinnamon.settings-daemon.peripherals.keyboard", "repeat-interval", _("Slow"), _("Fast"), 20, 2000, log=True, show_value=False, flipped=True)
             settings.add_reveal_row(slider, "org.cinnamon.settings-daemon.peripherals.keyboard", "repeat")
 
             settings = page.add_section(_("Text cursor"))
@@ -233,7 +239,7 @@ class Module:
             switch = GSettingsSwitch(_("Text cursor blinks"), "org.cinnamon.desktop.interface", "cursor-blink")
             settings.add_row(switch)
 
-            slider = GSettingsRange(_("Blink speed:"), "org.cinnamon.desktop.interface", "cursor-blink-time", _("Slow"), _("Fast"), 100, 2500, invert=True)
+            slider = GSettingsRange(_("Blink speed:"), "org.cinnamon.desktop.interface", "cursor-blink-time", _("Slow"), _("Fast"), 100, 2500, show_value=False, flipped=True)
             settings.add_reveal_row(slider, "org.cinnamon.desktop.interface", "cursor-blink")
 
             # vbox.add(Gtk.Label.new(_("Test Box")))
@@ -250,7 +256,7 @@ class Module:
             headingbox.pack_end(Gtk.Label.new(_("To edit a keyboard binding, click it and press the new keys, or press backspace to clear it.")), False, False, 1)
 
             paned = Gtk.Paned(orientation = Gtk.Orientation.HORIZONTAL)
-            Gtk.StyleContext.add_class(Gtk.Widget.get_style_context(paned), "wide")
+            paned.set_wide_handle(True)
 
             left_vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 2)
             right_vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 2)
@@ -345,7 +351,7 @@ class Module:
             entry_column.set_alignment(.5)
             self.entry_tree.append_column(entry_column)
 
-            self.entry_tree.set_tooltip_text("%s\n%s\n%s" % (_("Click to set a new accelerator key."), _("Press Escape or click again to cancel the operation."), _("Press Backspace to clear the existing keybinding.")))
+            self.entry_tree.set_tooltip_text(CellRendererKeybinding.TOOLTIP_TEXT)
 
             self.main_store = []
 
@@ -459,17 +465,15 @@ class Module:
             for keybinding in category.keybindings:
                 for entry in keybinding.entries:
                     found = False
-                    if accel_string.lower() == entry.lower():
-                        found = True
-                    elif accel_string.replace("<Primary>", "<Control>").lower() == entry.lower():
+                    if Gtk.accelerator_parse_with_keycode(accel_string) == Gtk.accelerator_parse_with_keycode(entry):
                         found = True
 
                     if found and keybinding.label != current_keybinding.label:
                         dialog = Gtk.MessageDialog(None,
-                                    Gtk.DialogFlags.DESTROY_WITH_PARENT,
-                                    Gtk.MessageType.QUESTION,
-                                    Gtk.ButtonsType.YES_NO,
-                                    None)
+                                                   Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                                                   Gtk.MessageType.QUESTION,
+                                                   Gtk.ButtonsType.YES_NO,
+                                                   None)
                         dialog.set_default_size(400, 200)
                         msg = _("This key combination, <b>%(combination)s</b> is currently in use by <b>%(old)s</b>.  ")
                         msg += _("If you continue, the combination will be reassigned to <b>%(new)s</b>.\n\n")
@@ -740,10 +744,10 @@ class AddCustomDialog(Gtk.Dialog):
         else:
             ok_button_label = _("Add")
         super(AddCustomDialog, self).__init__(_("Add custom shortcut"),
-                                                None,
-                                                0,
-                                                (ok_button_label, Gtk.ResponseType.OK,
-                                                _("Cancel"), Gtk.ResponseType.CANCEL))
+                                              None,
+                                              0,
+                                              (ok_button_label, Gtk.ResponseType.OK,
+                                               _("Cancel"), Gtk.ResponseType.CANCEL))
         self.set_default_size(350, 100)
         name_box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 2)
         command_box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 2)
