@@ -1,92 +1,121 @@
 // -*- indent-tabs-mode: nil -*-
-const Cinnamon = imports.gi.Cinnamon;
 const Clutter = imports.gi.Clutter;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const St = imports.gi.St;
 const Meta = imports.gi.Meta;
 const Mainloop = imports.mainloop;
+const Lang = imports.lang;
 
 const Desklet = imports.ui.desklet;
 const DND = imports.ui.dnd;
 const Extension = imports.ui.extension;
 const Main = imports.ui.main;
+const {getModuleByIndex} = imports.misc.fileUtils;
+const {queryCollection} = imports.misc.util;
 
-// Maps uuid -> metadata object
-var deskletMeta;
 // Maps uuid -> importer object (desklet directory tree)
 var desklets;
-// Maps uuid -> desklet objects
-var deskletObj = {};
+// Kept for compatibility
+var deskletMeta;
 
-var enabledDeskletDefinitions;
+var rawDefinitions;
+var definitions = [];
 
 var deskletsLoaded = false;
 
 var deskletsDragging = false;
 
-let userDeskletsDir;
+var userDeskletsDir;
 
-let mouseTrackEnabled = false;
-let mouseTrackTimoutId = 0;
+var mouseTrackEnabled = false;
+var mouseTrackTimoutId = 0;
+var promises = [];
 
+var deskletChangeKey = 0;
 const ENABLED_DESKLETS_KEY = 'enabled-desklets';
 const DESKLET_SNAP_KEY = 'desklet-snap';
 const DESKLET_SNAP_INTERVAL_KEY = 'desklet-snap-interval';
+
+function initEnabledDesklets() {
+    for (let i = 0; i < definitions.length; i++) {
+        promises.push(Extension.loadExtension(definitions[i].uuid, Extension.Type.DESKLET))
+    }
+    return Promise.all(promises).then(function() {
+        promises = [];
+    });
+}
+
+function unloadRemovedDesklets(removedDeskletUUIDs) {
+    for (let i = 0; i < removedDeskletUUIDs.length; i++) {
+        Extension.unloadExtension(removedDeskletUUIDs[i], Extension.Type.DESKLET);
+    }
+}
+
 /**
  * init:
  *
  * Initialize desklet manager
  */
 function init(){
-    desklets = Extension.Type.DESKLET.maps.importObjects;
-    deskletMeta = Extension.Type.DESKLET.maps.meta;
-
+    let startTime = new Date().getTime();
+    try {
+        desklets = imports.desklets;
+    } catch (e) {
+        desklets = {};
+    }
+    deskletMeta = Extension.Type.DESKLET.legacyMeta;
     deskletsLoaded = false
 
-    enabledDeskletDefinitions = getEnabledDeskletDefinitions();
-    let hasDesklets = false;
-    for (let uuid in enabledDeskletDefinitions.uuidMap) {
-        if(Extension.loadExtension(uuid, Extension.Type.DESKLET))
-            hasDesklets = true;
-    }
+    definitions = getDefinitions();
 
-    global.settings.connect('changed::' + ENABLED_DESKLETS_KEY, _onEnabledDeskletsChanged);
-    global.settings.connect('changed::' + DESKLET_SNAP_KEY, _onDeskletSnapChanged);
-    global.settings.connect('changed::' + DESKLET_SNAP_INTERVAL_KEY, _onDeskletSnapChanged);
+    return initEnabledDesklets().then(function() {
+        deskletChangeKey = global.settings.connect('changed::' + ENABLED_DESKLETS_KEY, _onEnabledDeskletsChanged);
+        global.settings.connect('changed::' + DESKLET_SNAP_KEY, _onDeskletSnapChanged);
+        global.settings.connect('changed::' + DESKLET_SNAP_INTERVAL_KEY, _onDeskletSnapChanged);
 
-    deskletsLoaded = true;
-    enableMouseTracking(true);
+        deskletsLoaded = true;
+        enableMouseTracking(true);
+        global.log(`DeskletManager started in ${new Date().getTime() - startTime} ms`);
+    });
+}
+
+function getDeskletDefinition(definition) {
+    return queryCollection(definitions, definition);
 }
 
 function enableMouseTracking(enable) {
-    if(enable && !mouseTrackTimoutId)
+    if (enable && !mouseTrackTimoutId) {
         mouseTrackTimoutId = Mainloop.timeout_add(500, checkMouseTracking);
-    else if (!enable && mouseTrackTimoutId) {
+    } else if (!enable && mouseTrackTimoutId) {
         Mainloop.source_remove(mouseTrackTimoutId);
         mouseTrackTimoutId = 0;
-        for(let desklet_id in deskletObj) {
-            deskletObj[desklet_id]._untrackMouse();
+
+        for (let i = 0; i < definitions.length; i++) {
+            if (definitions[i].desklet) {
+                definitions[i].desklet._untrackMouse();
+            }
         }
     }
 }
 
 function hasMouseWindow(){
     let window = global.screen.get_mouse_window(null);
-    return window && window.window_type != Meta.WindowType.DESKTOP;
+    return window && window.window_type !== Meta.WindowType.DESKTOP;
 }
 
 function checkMouseTracking() {
     let enable = !hasMouseWindow();
-    if(mouseTrackEnabled != enable) {
+    if (mouseTrackEnabled !== enable) {
         mouseTrackEnabled = enable;
-        if(enable) {
-            for(let desklet_id in deskletObj) {
-                deskletObj[desklet_id]._trackMouse();
+        for (let i = 0; i < definitions.length; i++) {
+            if (!definitions[i].desklet) {
+                continue;
             }
-        } else {
-            for(let desklet_id in deskletObj) {
-                deskletObj[desklet_id]._untrackMouse();
+            if (enable) {
+                definitions[i].desklet._trackMouse();
+            } else {
+                definitions[i].desklet._untrackMouse();
             }
         }
     }
@@ -97,7 +126,7 @@ function checkMouseTracking() {
  * removeDesklet:
  * @uuid (string): uuid of the desklet
  * @desklet_id (int): id of the desklet
- * 
+ *
  * Disable and remove the desklet @uuid:@desklet_id
  */
 function removeDesklet(uuid, desklet_id){
@@ -111,47 +140,39 @@ function removeDesklet(uuid, desklet_id){
 }
 
 /**
- * getEnabledDeskletDefinitons:
+ * getDefinitons:
  *
  * Gets the list of enabled desklets. Returns an associative array of three items:
  * raw: the unprocessed array from gsettings
- * uuidMap: maps uuid -> list of desklet definitions
- * idMap: maps desklet_id -> single desklet definition
+ * definitions: Array(Object)
  *
  * Returns (dictionary): Associative array of three items
  */
-function getEnabledDeskletDefinitions() {
-    let result = {
-        // the raw list from gsettings
-        raw: global.settings.get_strv(ENABLED_DESKLETS_KEY),
-        // maps uuid -> list of desklet definitions
-        uuidMap: {},
-        // maps desklet_id -> single desklet definition
-        idMap: {}
-    };
+function getDefinitions() {
+    let _definitions = [];
+    rawDefinitions = global.settings.get_strv(ENABLED_DESKLETS_KEY);
 
     // Parse all definitions
-    for (let i=0; i<result.raw.length; i++) {
-        let deskletDefinition = _getDeskletDefinition(result.raw[i]);
-        if(deskletDefinition) {
-            if(!result.uuidMap[deskletDefinition.uuid])
-                result.uuidMap[deskletDefinition.uuid] = [];
-            result.uuidMap[deskletDefinition.uuid].push(deskletDefinition);
-            result.idMap[deskletDefinition.desklet_id] = deskletDefinition;
+    for (let i=0; i < rawDefinitions.length; i++) {
+        let deskletDefinition = createDeskletDefinition(rawDefinitions[i]);
+        if (deskletDefinition) {
+            _definitions.push(deskletDefinition);
         }
     }
 
-    return result;
+    return _definitions;
 }
 
 // Callback for extension.js
-function finishExtensionLoad(extension) {
+function finishExtensionLoad(extensionIndex) {
     // Add all desklet instances for this extension
-    let definitions = enabledDeskletDefinitions.uuidMap[extension.uuid];
-    if (definitions) {
-        for(let i=0; i<definitions.length; i++) {
-            if (!_loadDesklet(extension, definitions[i]))
-                return false;
+    let extension = Extension.extensions[extensionIndex];
+    for (let i = 0; i < definitions.length; i++) {
+        if (definitions[i].uuid !== extension.uuid) {
+            continue;
+        }
+        if (!_loadDesklet(extension, definitions[i])) {
+            return false;
         }
     }
     return true;
@@ -160,69 +181,79 @@ function finishExtensionLoad(extension) {
 // Callback for extension.js
 function prepareExtensionUnload(extension, deleteConfig) {
     // Remove all desklet instances for this extension
-    for(let desklet_id in extension._loadedDefinitions) {
-        _unloadDesklet(extension._loadedDefinitions[desklet_id], deleteConfig);
+    for (let i = 0; i < definitions.length; i++) {
+        definitions[i]
+        if (extension.uuid !== definitions[i].uuid) {
+            continue;
+        }
+        _unloadDesklet(definitions[i], deleteConfig);
     }
 }
 
-function _onEnabledDeskletsChanged(){
-    try{
-        let newEnabledDeskletDefinitions = getEnabledDeskletDefinitions();
-        // Remove all desklet instances that do not exist in the definition anymore.
-        for (let desklet_id in enabledDeskletDefinitions.idMap) {
-            if(!newEnabledDeskletDefinitions.idMap[desklet_id]) {
-                _unloadDesklet(enabledDeskletDefinitions.idMap[desklet_id], true);
-            }
+function _onEnabledDeskletsChanged() {
+    let oldDefinitions = definitions.slice();
+    definitions = getDefinitions();
+    let addedDesklets = [];
+    let removedDesklets = [];
+    let unChangedDesklets = [];
+
+    for (let i = 0; i < definitions.length; i++) {
+        let {uuid, desklet_id} = definitions[i];
+        let oldDefinition = queryCollection(oldDefinitions, {uuid, desklet_id});
+
+        let isEqualToOldDefinition = _deskletDefinitionsEqual(definitions[i], oldDefinition);
+
+        if (oldDefinition && !isEqualToOldDefinition) {
+            removedDesklets.push({changed: true, definition: oldDefinition});
         }
 
-        // Unload all desklet extensions that do not exist in the definition anymore.
-        for (let uuid in enabledDeskletDefinitions.uuidMap) {
-            if(!newEnabledDeskletDefinitions.uuidMap[uuid]) {
-                Extension.unloadExtension(uuid, Extension.Type.DESKLET);
-            }
-        }
-        // Add or move desklet instances of already loaded desklet extensions
-        for (let desklet_id in newEnabledDeskletDefinitions.idMap) {
-            let newDef = newEnabledDeskletDefinitions.idMap[desklet_id];
-            let oldDef = enabledDeskletDefinitions.idMap[desklet_id];
-
-            if(!oldDef || !_deskletDefinitionsEqual(newDef, oldDef)) {
-                let extension = Extension.Type.DESKLET.maps.objects[newDef.uuid];
-                if(extension) {
-                    _loadDesklet(extension, newDef);
-                }
-            }
+        if (!oldDefinition || !isEqualToOldDefinition) {
+            let extension = Extension.getExtension(uuid);
+            addedDesklets.push({extension, definition: definitions[i]});
+            continue;
         }
 
-        enabledDeskletDefinitions = newEnabledDeskletDefinitions;
-
-        // Make sure all desklet extensions are loaded.
-        // Once loaded, the desklets will add themselves via finishExtensionLoad
-        let hasDesklets = false;
-        for (let uuid in enabledDeskletDefinitions.uuidMap) {
-            if(Extension.loadExtension(uuid, Extension.Type.DESKLET))
-                hasDesklets = true;
-        }
-
-    } catch (e) {
-        global.logError('Failed to refresh list of desklets', e);
+        unChangedDesklets.push(desklet_id);
     }
+    for (let i = 0; i < oldDefinitions.length; i++) {
+        if (unChangedDesklets.indexOf(oldDefinitions[i].desklet_id) === -1) {
+            removedDesklets.push({changed: false, definition: oldDefinitions[i]});
+        }
+    }
+    for (let i = 0; i < removedDesklets.length; i++) {
+        let {uuid} = removedDesklets[i].definition;
+        _unloadDesklet(
+            removedDesklets[i].definition,
+            Extension.get_max_instances(uuid, Extension.Type.DESKLET) !== 1 && !removedDesklets[i].changed
+        );
+    }
+    for (let i = 0; i < addedDesklets.length; i++) {
+        let {extension, definition} = addedDesklets[i];
+        if (!extension) {
+            continue;
+        }
+        _loadDesklet(extension, definition);
+    }
+
+    // Make sure all desklet extensions are loaded.
+    // Once loaded, the desklets will add themselves via finishExtensionLoad
+    initEnabledDesklets();
 }
 
 function _unloadDesklet(deskletDefinition, deleteConfig) {
-    let desklet = deskletObj[deskletDefinition.desklet_id];
-    if (desklet){
+    let {uuid, desklet_id} = deskletDefinition;
+    if (deskletDefinition.desklet) {
         try {
-            desklet.destroy(deleteConfig);
+            deskletDefinition.desklet.destroy(deleteConfig);
         } catch (e) {
-            global.logError("Failed to destroy desket: " + deskletDefinition.uuid + "/" + deskletDefinition.desklet_id, e);
+            global.logError("Failed to destroy desket: " + uuid + "/" + desklet_id, e);
         }
 
-        if (deleteConfig)
-            _removeDeskletConfigFile(deskletDefinition.uuid, deskletDefinition.desklet_id);
+        if (deleteConfig) {
+            _removeDeskletConfigFile(uuid, desklet_id);
+        }
 
-        delete desklet._extension._loadedDefinitions[deskletDefinition.desklet_id];
-        delete deskletObj[deskletDefinition.desklet_id];
+        deskletDefinition.desklet = null;
     }
 }
 
@@ -244,101 +275,105 @@ function _removeDeskletConfigFile(uuid, instanceId) {
 
 function _loadDesklet(extension, deskletDefinition) {
     // Try to lock the desklets role
-    if(!extension.lockRole(null))
+    if (!extension || !extension.lockRole(null))
         return false;
-    
+
     try {
         let desklet = _createDesklets(extension, deskletDefinition);
         if (!desklet)
             return false;
-        
+
         // Now actually lock the desklets role and set the provider
         if(!extension.lockRole(desklet))
             return false;
 
-        desklet._extension = extension;
-
         if (!Main.deskletContainer.contains(desklet.actor)) Main.deskletContainer.addDesklet(desklet.actor);
         desklet.actor.set_position(deskletDefinition.x, deskletDefinition.y);
 
-        if(!extension._loadedDefinitions) {
-            extension._loadedDefinitions = {};
-        }
-        extension._loadedDefinitions[deskletDefinition.desklet_id] = deskletDefinition;
-
         desklet.on_desklet_added_to_desktop_internal(deskletsLoaded && !deskletsDragging);
 
         deskletsDragging = false;
 
         return true;
     } catch (e) {
-        extension.logError('Failed to load desklet: ' + deskletDefinition.uuid + "/" + deskletDefinition.desklet_id, e);
+        Extension.logError('Failed to load desklet: ' + deskletDefinition.uuid + "/" + deskletDefinition.desklet_id, e);
         return false;
     }
 }
 
 function _createDesklets(extension, deskletDefinition) {
-    let desklet_id = deskletDefinition.desklet_id;
+    let {uuid, desklet_id} = deskletDefinition;
 
-    if (deskletObj[desklet_id]) {
+    if (deskletDefinition.desklet) {
         global.log(desklet_id + ' desklet already loaded');
-        return deskletObj[desklet_id];
+        return deskletDefinition.desklet;
     }
 
     let desklet;
     try {
-        desklet = extension.module.main(extension.meta, desklet_id);
+        desklet = getModuleByIndex(extension.moduleIndex).main(extension.meta, desklet_id);
     } catch (e) {
-        extension.logError('Failed to evaluate \'main\' function on desklet: ' + deskletDefinition.uuid + "/" + deskletDefinition.desklet_id, e);
+        Extension.logError('Failed to evaluate \'main\' function on desklet: ' + uuid + "/" + desklet_id, e);
         return null;
     }
 
-    deskletObj[desklet_id] = desklet;
+    deskletDefinition.desklet = desklet;
     desklet._uuid = extension.uuid;
-    desklet._meta = extension.meta; 
+    desklet._meta = extension.meta;
     desklet.instance_id = desklet_id;  // In case desklet constructor didn't set this
-    
+
     desklet.finalizeContextMenu();
-    
+
     return desklet;
 }
 
-function _getDeskletDefinition(definition) {
+function createDeskletDefinition(definition) {
     let elements = definition.split(":");
-    if (elements.length == 4) {
-        return {
-            uuid: elements[0],
-            desklet_id: elements[1],
-            x: elements[2],
-            y: elements[3]
-        };
-    } else {
+    if (elements.length !== 4) {
         global.logError("Bad desklet definition: " + definition);
         return null;
     }
+    let deskletDefinition = {
+        uuid: elements[0],
+        desklet_id: elements[1],
+        x: elements[2],
+        y: elements[3]
+    };
+
+    let existingDefinition = getDeskletDefinition(deskletDefinition);
+
+    if (existingDefinition) {
+        return existingDefinition;
+    }
+
+    deskletDefinition.desklet = null;
+
+    return deskletDefinition;
 }
 
 function _deskletDefinitionsEqual(a, b) {
-    return (a.uuid == b.uuid && a.x == b.x && a.y == b.y);
+    return (a && b && (a.uuid === b.uuid && a.x === b.x && a.y === b.y));
 }
 
 function get_object_for_instance (deskletId) {
-    if (deskletId in deskletObj) {
-        return deskletObj[deskletId];
-    } else {
+    let {desklet} = getDeskletDefinition({desklet_id: deskletId});
+    if (!desklet) {
         return null;
     }
+    return desklet;
 }
 
 function get_object_for_uuid (uuid, instanceId) {
-    for (let thisInstanceId in deskletObj) {
-        if (deskletObj[thisInstanceId]._uuid == uuid) {
-            if (instanceId == uuid || thisInstanceId == instanceId) {
-                return deskletObj[thisInstanceId]
-            }
-        }
+    let index = definitions.findIndex(function(definition) {
+        return (definition
+            && definition.desklet
+            && definition.desklet._uuid === uuid
+            && (definition.desklet_id === instanceId || instanceId === uuid));
+    });
+    if (index === -1) {
+        return null;
     }
-    return null;
+    return definitions[index].desklet;
 }
 
 function _onDeskletSnapChanged(){
@@ -363,7 +398,7 @@ function _onDeskletSnapChanged(){
 
 /**
  * #DeskletContainer
- * 
+ *
  * Container that contains manages all desklets actors
  */
 function DeskletContainer(){
@@ -375,17 +410,20 @@ DeskletContainer.prototype = {
         this.actor = new Clutter.Group();
         this.actor._delegate = this;
 
-        this.lastX = 0;
-        this.lastY = 0;
+        this.last_x = -1;
+        this.last_y = -1;
 
         this._dragPlaceholder = new St.Bin({style_class: 'desklet-drag-placeholder'});
         this._dragPlaceholder.hide();
+
+        this.isModal = false;
+        this.stageEventIds = [];
     },
 
     /**
      * addDesklet:
      * @actor (Clutter.Actor): actor of desklet to be added
-     * 
+     *
      * Adds @actor to the desklet container
      */
     addDesklet: function(actor){
@@ -395,7 +433,7 @@ DeskletContainer.prototype = {
     /**
      * contains:
      * @actor (Clutter.Actor): actor to be tested
-     * 
+     *
      * Whether the desklet container contains @actor
      *
      * Returns (boolean): whether the desklet container contains the actor
@@ -483,7 +521,10 @@ DeskletContainer.prototype = {
             }
         }
 
+        // We already moved this desklet, so skipping _onEnabledDeskletsChanged
+        global.settings.disconnect(deskletChangeKey);
         global.settings.set_strv(ENABLED_DESKLETS_KEY, enabledDesklets);
+        deskletChangeKey = global.settings.connect('changed::' + ENABLED_DESKLETS_KEY, _onEnabledDeskletsChanged);
 
         this._dragPlaceholder.hide();
         this.last_x = -1;
@@ -505,5 +546,60 @@ DeskletContainer.prototype = {
 
     hideDragPlaceholder: function() {
         this._dragPlaceholder.hide();
+    },
+
+    setModal: function() {
+        if (this.isModal) {
+            return;
+        }
+
+        this.stageEventIds = [
+            global.stage.connect('captured-event', Lang.bind(this, this.handleStageEvent)),
+            global.stage.connect('enter-event', Lang.bind(this, this.handleStageEvent)),
+            global.stage.connect('leave-event', Lang.bind(this, this.handleStageEvent))
+        ];
+
+        if (Main.pushModal(this.actor)) {
+            this.isModal = true;
+        }
+    },
+
+    unsetModal: function() {
+        if (!this.isModal) {
+            return;
+        }
+
+        for (let i = 0; i < this.stageEventIds.length; i++) {
+            global.stage.disconnect(this.stageEventIds[i]);
+        }
+        this.stageEventIds = [];
+
+        Main.popModal(this.actor);
+        this.isModal = false;
+    },
+
+    handleStageEvent: function(actor, event) {
+        let target = event.get_source();
+        let type = event.type();
+
+        if ((type === Clutter.EventType.BUTTON_PRESS || type === Clutter.EventType.BUTTON_RELEASE)
+            && target.toString().indexOf('ClutterStage') > -1) {
+            this.lower();
+        }
+
+        return false;
+    },
+
+    raise: function() {
+        if (this.actor.get_children().length === 0) {
+            return;
+        }
+        this.actor.get_parent().set_child_above_sibling(this.actor, null);
+        this.setModal();
+    },
+
+    lower: function() {
+        this.actor.get_parent().set_child_below_sibling(this.actor, global.window_group);
+        this.unsetModal();
     }
 };
